/*
 * TCS Dependency Map Renderer
 *
 * Copyright (c) 2005 TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDeveloper
 */

using System;
using System.IO;
using NUnit.Framework;

namespace TopCoder.Rules.DependencyMap.View
{

    /// <summary>
    /// Unit test fixture for DependencyRenderer.
    ///
    /// Testing of DependencyRenderer only handles the properties. The accuracy of the rendering algorithm
    /// is tested in the FixedGridLayoutManager.
    /// </summary>
    [TestFixture]
    public class DependencyRendererUnitTests
    {

        /// <summary>
        /// The filename to persist to.
        /// </summary>
        private string filename = @"../../test_files/unittest.svg";

        /// <summary>
        /// The shape factory to use.
        /// </summary>
        private ShapeFactory factory = null;

        /// <summary>
        /// The diagram renderer to use.
        /// </summary>
        private IDiagramRenderer dr = null;

        /// <summary>
        /// A second diagram renderer to use.
        /// </summary>
        private IDiagramRenderer dr2 = null;

        /// <summary>
        /// The layout manager to use.
        /// </summary>
        private ILayoutManager manager = new FixedGridLayoutManager();

        /// <summary>
        /// The dependency renderer instance to test.
        /// </summary>
        private DependencyRenderer renderer = null;

        /// <summary>
        /// Create instance of shape factory, diagram renderer and dependency renderer for each test.
        /// </summary>
        [SetUp]
        protected void SetUp()
        {
            factory = new ShapeFactory();
            dr = new SVGRenderer(filename);
            dr2 = new SVGRenderer(filename);
            renderer = new DependencyRenderer(factory);
        }

        /// <summary>
        /// Create instance.
        /// </summary>
        [Test]
        public void Create()
        {
            renderer = new DependencyRenderer();
            Assert.IsNotNull(renderer.LayoutManager, "Layout manager not set.");
            Assert.AreEqual(0, renderer.DiagramRenderers.Count, "Incorrect renderer count.");
        }

        /// <summary>
        /// Create instance with shape factory.
        /// </summary>
        [Test]
        public void Create_ShapeFactory()
        {
            renderer = new DependencyRenderer(factory);
            Assert.IsNotNull(renderer.LayoutManager, "Layout manager not set.");
            Assert.AreEqual(0, renderer.DiagramRenderers.Count, "Incorrect renderer count.");
        }

        /// <summary>
        /// Create instance with null shape factory.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void Create_NullShapeFactory()
        {
            new DependencyRenderer(null);
        }

        /// <summary>
        /// Test AddDiagramRenderer().
        /// </summary>
        [Test]
        public void AddDiagramRenderer()
        {
            renderer.AddDiagramRenderer(dr);

            Assert.AreEqual(1, renderer.DiagramRenderers.Count, "Incorrect renderer count.");
            Assert.IsTrue(renderer.DiagramRenderers.Contains(dr), "Does not contain the renderer.");

            renderer.AddDiagramRenderer(dr2);

            Assert.AreEqual(2, renderer.DiagramRenderers.Count, "Incorrect renderer count.");
            Assert.IsTrue(renderer.DiagramRenderers.Contains(dr2), "Does not contain the renderer.");
        }

        /// <summary>
        /// Test addDiagramRenderer() with null diagram renderer.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void AddDiagramRenderer_NullDiagramRenderer()
        {
            renderer.AddDiagramRenderer(null);
        }

        /// <summary>
        /// Test RemoveDiagramRenderer().
        /// </summary>
        [Test]
        public void RemoveDiagramRenderer()
        {
            renderer.AddDiagramRenderer(dr);
            renderer.AddDiagramRenderer(dr2);

            renderer.RemoveDiagramRenderer(dr);
            Assert.AreEqual(1, renderer.DiagramRenderers.Count, "Incorrect renderer count.");

            renderer.RemoveDiagramRenderer(dr2);
            Assert.AreEqual(0, renderer.DiagramRenderers.Count, "Incorrect renderer count.");
        }

        /// <summary>
        /// Test RemoveDiagramRenderer() with null diagram renderer.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void RemoveDiagramRenderer_NullDiagramRenderer()
        {
            renderer.RemoveDiagramRenderer(null);
        }

        /// <summary>
        /// Test getter DiagramRenderers.
        /// </summary>
        [Test]
        public void GetterDiagramRenderers()
        {
            renderer.AddDiagramRenderer(dr);
            renderer.AddDiagramRenderer(dr2);

            Assert.AreEqual(2, renderer.DiagramRenderers.Count, "Incorrect renderer count.");
            Assert.IsTrue(renderer.DiagramRenderers.Contains(dr), "Does not contain the renderer.");
            Assert.IsTrue(renderer.DiagramRenderers.Contains(dr2), "Does not contain the renderer.");
        }

        /// <summary>
        /// Test setter LayoutManager.
        /// </summary>
        [Test]
        public void SetterLayoutManager()
        {
            renderer.LayoutManager = manager;
            Assert.AreEqual(manager, renderer.LayoutManager, "Layout manager mismatches.");
        }

        /// <summary>
        /// Test setter LayoutManager with null layout manager.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void SetterLayoutManager_NullLayoutManager()
        {
            renderer.LayoutManager = null;
        }

        /// <summary>
        /// Test getter LayoutManager.
        /// </summary>
        [Test]
        public void GetterLayoutManager()
        {
            renderer.LayoutManager = manager;
            Assert.AreEqual(manager, renderer.LayoutManager, "Layout manager mismatches.");
        }

        /// <summary>
        /// Test Render().
        /// </summary>
        [Test]
        public void Render()
        {
            renderer.AddDiagramRenderer(dr);
            renderer.AddDiagramRenderer(dr2);
            renderer.Render(new Dependency("Dependency Map Renderer"));

            Assert.IsTrue(File.Exists(filename), "Output does not exist.");
            File.Delete(filename);
        }

        /// <summary>
        /// Test Render() with null dependency.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void Render_NullDependency()
        {
            renderer.Render(null);
        }

    }

}
