/**
 * TCS Dependency Map Renderer
 *
 * Copyright (c) 2005 TopCoder, Inc. All rights Reserved.
 */
package com.topcoder.rules.dependencymap.view;

import java.awt.Color;
import java.awt.Font;
import java.io.File;

import junit.framework.TestCase;

/**
 * Unit test cases for FixedGridLayoutManager.
 *
 * Testing of FixedGridLayoutManager includes the accuracy testing of the layout algorithm,
 * to verify that each parameter works independently.
 *
 * @author TCSDeveloper
 */
public class FixedGridLayoutManagerUnitTests extends TestCase {

    /**
     * The filename to persist to.
     */
    private String filename = "test_files/unittests/unittest-";

    /**
     * FixedGridLayoutManager instance to test.
     */
    private FixedGridLayoutManager manager = null;

    /**
     * ShapeFactory instance to use.
     */
    private ShapeFactory factory = null;

    /**
     * RendererGraphics to use.
     */
    //private RendererGraphics graphics = null;

    /**
     * Create new layout manager, shape factory and renderer graphics for each test.
     */
    protected void setUp() {
        manager = new FixedGridLayoutManager();
        factory = new ShapeFactory();        
    }

    /**
     * Create instance.
     */
    public void testCreate() {
        manager = new FixedGridLayoutManager();
    }

    /**
     * Layout single dependency.
     *
     * @throws RendererException to JUnit.
     */
    public void testLayout_SingleDependency() throws RendererException {
        Dependency dependency = new Dependency("Dependency Map Renderer");
        
        SVGRenderer renderer = new SVGRenderer(new File(filename + "single_dependency.svg"));
        
        manager.layout(dependency, factory, renderer);
    }

    /**
     * Layout dependency tree.
     *
     * @throws RendererException to JUnit.
     */
    public void testLayout_DependencyTree() throws RendererException {
        Dependency dependency = new Dependency("Dependency Map Renderer");
        dependency.addDependant(new Dependency("SVG Generator"));
        dependency.addDependant(new Dependency("Base Exception"));
        
        SVGRenderer renderer = new SVGRenderer(new File(filename + "dependency_tree.svg"));
        
        manager.layout(dependency, factory, renderer);
    }

    /**
     * Layout seen dependency.
     *
     * @throws RendererException to JUnit.
     */
    public void testLayout_SeenDependency() throws RendererException {
        Dependency dependency = new Dependency("Dependency Map Renderer");
        dependency.addDependant(dependency);

        SVGRenderer renderer = new SVGRenderer(new File(filename + "seen_dependency.svg"));

        manager.layout(dependency, factory, renderer);
    }

    /**
     * Layout shapeOverride.
     *
     * @throws RendererException to JUnit.
     */
    public void testLayout_ShapeOverride() throws RendererException {
        Dependency dependency = new Dependency("Dependency Map Renderer", ShapeFactory.SEEN_ELEMENT);
        
        SVGRenderer renderer = new SVGRenderer(new File(filename + "shape_override.svg"));
        
        manager.layout(dependency, factory, renderer);
    }

    /**
     * Layout colPadding and rowPadding.
     *
     * @throws RendererException to JUnit.
     */
    public void testLayout_ColPaddingRowPadding() throws RendererException {
        manager = new FixedGridLayoutManager();
        Dependency dependency = new Dependency("Dependency Map Renderer");
        dependency.addDependant(new Dependency("SVG Generator"));
        dependency.addDependant(new Dependency("Base Exception"));

        SVGRenderer renderer = new SVGRenderer(new File(filename + "colpadding_rowpadding.svg"));

        manager.layout(dependency, factory, renderer);
    }

    /**
     * Layout textMargin.
     *
     * @throws RendererException to JUnit.
     */
    public void testLayout_TextMargin() throws RendererException {
        manager = new FixedGridLayoutManager();
        Dependency dependency = new Dependency("Dependency Map Renderer");
        dependency.addDependant(new Dependency("SVG Generator"));
        dependency.addDependant(new Dependency("Base Exception"));

        SVGRenderer renderer = new SVGRenderer(new File(filename + "text_margin.svg"));

        manager.layout(dependency, factory, renderer);
    }

    /**
     * Test layout() with null dependency.
     * @throws RendererException to JUnit.
     */
    public void testLayout_NullDependency() throws RendererException {
        try {

            SVGRenderer renderer = new SVGRenderer(new File(filename + "fail_layout.svg"));
            manager.layout(null, factory, renderer);
            fail("Should have thrown NullPointerException.");
        } catch (NullPointerException npe) {
        }
    }

    /**
     * Test layout() with null shape factory.
     * @throws RendererException to Junit.
     */
    public void testLayout_NullShapeFactory() throws RendererException {
        try {
            SVGRenderer renderer = new SVGRenderer(new File(filename + "fail_factory.svg"));
            manager.layout(new Dependency("a"), null, renderer);
            fail("Should have thrown NullPointerException.");
        } catch (NullPointerException npe) {
        }
    }

    /**
     * Test layout() with null renderer graphics.
     * @throws RendererException to JUnit.
     */
    public void testLayout_NullRendererGraphics() throws RendererException {
        try {
            manager.layout(new Dependency("a"), factory, null);
            fail("Should have thrown NullPointerException.");
        } catch (NullPointerException npe) {
        }
    }
    
    /**
     * Test layout() with complex sample 1
     * 
     * @throws RendererException to Junit.
     */
    public void testLayout_1() throws RendererException {
        Dependency[] d = new Dependency['q'];
        for (char i = 'a'; i <= 'p'; i++) {
            d[i] = new Dependency("" + Character.toUpperCase(i)); 
        }
        d['a'].addDependant(d['i']);
        d['a'].addDependant(d['b']);
        d['b'].addDependant(d['c']);
        d['b'].addDependant(d['f']);
        d['b'].addDependant(d['g']);
        d['b'].addDependant(d['h']);
        d['c'].addDependant(d['d']);
        d['c'].addDependant(d['e']);
        d['i'].addDependant(d['j']);
        d['i'].addDependant(d['k']);
        d['i'].addDependant(d['l']);
        d['i'].addDependant(d['m']);
        d['m'].addDependant(d['n']);
        d['m'].addDependant(d['o']);
        d['m'].addDependant(d['p']);
        SVGRenderer renderer = new SVGRenderer(new File(filename + "complex_1.svg"));
        manager.layout(d['a'], factory, renderer);
    }
    
    /**
     * Test layout() with complex sample 2
     * 
     * @throws RendererException to Junit.
     */
    public void testLayout_2() throws RendererException {
        Dependency[] d = new Dependency['l'];
        for (char i = 'a'; i <= 'k'; i++) {
            d[i] = new Dependency("" + Character.toUpperCase(i)); 
        }
        d['a'].addDependant(d['b']);
        d['b'].addDependant(d['c']);
        d['b'].addDependant(d['d']);
        d['b'].addDependant(d['k']);
        d['d'].addDependant(d['e']);
        d['d'].addDependant(d['h']);
        d['d'].addDependant(d['i']);
        d['d'].addDependant(d['j']);
        d['e'].addDependant(d['f']);
        d['e'].addDependant(d['g']);
        SVGRenderer renderer = new SVGRenderer(new File(filename + "complex_2.svg"));
        manager.layout(d['a'], factory, renderer);
    }
    
    /**
     * Test getColPadding()
     *
     */
    public void testGetColPadding() {
        for (int i = 1; i < 100; i++) {
            manager.setColPadding(i);
            assertEquals("Incorrect result", i, manager.getColPadding());
        }
    }

    /**
     * Test setColPadding()
     *
     */
    public void testSetColPadding() {
        for (int i = 1; i < 100; i++) {
            manager.setColPadding(i);
            assertEquals("Incorrect result", i, manager.getColPadding());
        }
    }

    /**
     * Test getRowPadding()
     *
     */
    public void testGetRowPadding() {
        for (int i = 1; i < 100; i++) {
            manager.setRowPadding(i);
            assertEquals("Incorrect result", i, manager.getRowPadding());
        }
    }

    /**
     * Test setRowPadding()
     *
     */
    public void testSetRowPadding() {
        for (int i = 1; i < 100; i++) {
            manager.setRowPadding(i);
            assertEquals("Incorrect result", i, manager.getRowPadding());
        }
    }
    
    /**
     * Test getTextMargin()
     *
     */
    public void testGetTextMargin() {
        for (int i = 1; i < 100; i++) {
            manager.setTextMargin(i);
            assertEquals("Incorrect result", i, manager.getTextMargin());
        }
    }

    /**
     * Test setTextMargin()
     *
     */
    public void testSetTextMargin() {
        for (int i = 1; i < 100; i++) {
            manager.setTextMargin(i);
            assertEquals("Incorrect result", i, manager.getTextMargin());
        }
    }
    
    /**
     * Test getShapeFillColor()
     *
     */
    public void testGetShapeFillColor() {
        for (int i = 1; i < 100; i++) {
            Color c = new Color(i);
            manager.setShapeFillColor(c);
            assertEquals("Incorrect result", c, manager.getShapeFillColor());
        }
    }

    /**
     * Test setShapeFillColor()
     *
     */
    public void testSetShapeFillColor() {
        for (int i = 1; i < 100; i++) {
            Color c = new Color(i);
            manager.setShapeFillColor(c);
            assertEquals("Incorrect result", c, manager.getShapeFillColor());
        }
    }
    
    /**
     * Test getShapeLineColor()
     *
     */
    public void testGetShapeLineColor() {
        for (int i = 1; i < 100; i++) {
            Color c = new Color(i);
            manager.setShapeLineColor(c);
            assertEquals("Incorrect result", c, manager.getShapeLineColor());
        }
    }

    /**
     * Test setShapeLineColor()
     *
     */
    public void testSetShapeLineColor() {
        for (int i = 1; i < 100; i++) {
            Color c = new Color(i);
            manager.setShapeLineColor(c);
            assertEquals("Incorrect result", c, manager.getShapeLineColor());
        }
    }
    
    /**
     * Test getArrowFillColor()
     *
     */
    public void testGetArrowFillColor() {
        for (int i = 1; i < 100; i++) {
            Color c = new Color(i);
            manager.setArrowFillColor(c);
            assertEquals("Incorrect result", c, manager.getArrowFillColor());
        }
    }

    /**
     * Test setArrowFillColor()
     *
     */
    public void testSetArrowFillColor() {
        for (int i = 1; i < 100; i++) {
            Color c = new Color(i);
            manager.setArrowFillColor(c);
            assertEquals("Incorrect result", c, manager.getArrowFillColor());
        }
    }

    /**
     * Test getArrowLineColor()
     *
     */
    public void testGetArrowLineColor() {
        for (int i = 1; i < 100; i++) {
            Color c = new Color(i);
            manager.setArrowLineColor(c);
            assertEquals("Incorrect result", c, manager.getArrowLineColor());
        }
    }

    /**
     * Test setArrowLineColor()
     *
     */
    public void testSetArrowLineColor() {
        for (int i = 1; i < 100; i++) {
            Color c = new Color(i);
            manager.setArrowLineColor(c);
            assertEquals("Incorrect result", c, manager.getArrowLineColor());
        }
    }    
    
    /**
     * Test getTextColor()
     *
     */
    public void testGetTextColor() {
        for (int i = 1; i < 100; i++) {
            Color c = new Color(i);
            manager.setTextColor(c);
            assertEquals("Incorrect result", c, manager.getTextColor());
        }
    }

    /**
     * Test setTextColor()
     *
     */
    public void testSetTextColor() {
        for (int i = 1; i < 100; i++) {
            Color c = new Color(i);
            manager.setTextColor(c);
            assertEquals("Incorrect result", c, manager.getTextColor());
        }
    }
    
    /**
     * Test getTextFont()
     *
     */
    public void testGetTextFont() {
        Font f = new Font("SansSerif", Font.PLAIN, 12);
        manager.setTextFont(f);
        assertEquals("Incorrect result", f, manager.getTextFont());
    }

    /**
     * Test setTextFont()
     *
     */
    public void testSetTextFont() {
        Font f = new Font("SansSerif", Font.PLAIN, 12);
        manager.setTextFont(f);
        assertEquals("Incorrect result", f, manager.getTextFont());
    }        
}